/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  Bullet.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Thu Apr 08 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "Bullet.h"
#include "YourShip.h"
#include "point3d.h"
#include "vector3d.h"

Bullet::Bullet(GLfloat *params, YourShip *own) // start coord (3), velocity(3)
:Object(kBulletObject, kHomingMineObject|kMineObject|kShipObject)
{
	for (int x = 0; x < 6; x++)
		param[x] = params[x];
	elapsedTime = 0;
	owner = own;
}

void Bullet::draw()
{
	glColor3f(1.0, 0, 0);
	drawSphere(0.02, 4);
	/*
	glBegin(GL_LINES);
	 glColor3f(1.0, 0, 0);
	glVertex3f(param[0], param[1], param[2]);
	glVertex3f(param[0]+.2*param[3], param[1]+.2*param[4], param[2]+.2*param[5]);
	glEnd();
*/
}

uint32_t Bullet::stepTime(double amount)
{
	elapsedTime += amount;
	for (int x = 0; x < 3; x++)
		param[x]+=amount*param[x+3];
	if (elapsedTime > 10)
		return kObjectDestroyed;
	return kObjectRemains;
}

point3d Bullet::getLocation()
{
	point3d p;
	p.x = param[0];
	p.y = param[1];
	p.z = param[2];
	return p;
}

void Bullet::drawSphere(double tRadius, int precision)
{
  int i,j;
	int n = precision;
	double r = tRadius;
  double theta1,theta2,theta3;
  point3d e,p,c(0, 0, 0);
  
 	glTranslatef(param[0], param[1], param[2]);
	if (r < 0)
		r = -r;
	if (n < 0)
		n = -n;
	if (n < 4 || r <= 0) {
		glBegin(GL_POINTS);
		glVertex3f(c.x,c.y,c.z);
		glEnd();
	}
	else {
		for (j=0/*n/4*/;j<n/2;j++) {
			theta1 = j * TWOPI / n - PID2;
			theta2 = (j + 1) * TWOPI / n - PID2;
			
			//glBegin(GL_QUAD_STRIP);
			//glBegin(GL_TRIANGLE_STRIP);
			glBegin(GL_LINE_STRIP);
			for (i=0;i<=n;i++) {
				theta3 = i * TWOPI / n;
				
				e.x = cos(theta2) * cos(theta3);
				e.y = sin(theta2);
				e.z = cos(theta2) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;
				
				glNormal3f(-e.x,-e.y,-e.z);
				//glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
				glVertex3f(p.x,p.y,p.z);
				
				e.x = cos(theta1) * cos(theta3);
				e.y = sin(theta1);
				e.z = cos(theta1) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;
				
				glNormal3f(-e.x,-e.y,-e.z);
				//glTexCoord2f(i/(double)n,2*j/(double)n);
				glVertex3f(p.x,p.y,p.z);
			}
			glEnd();
		}
	}
 	glTranslatef(-param[0], -param[1], -param[2]);
}

double Bullet::getRadius()
{
	return 0.02;
}

uint32_t Bullet::testCollision(Object *o)
{
	if (o == owner) // bullets can't kill what they came from
		return kNothingDestroyed;
	vector3d p1 = getLocation().getVector(o->getLocation());
	float rad = getRadius()+o->getRadius();
	if (p1.squaredLength() < rad*rad)
  {
    if (owner)
    {
      if (o->getType() == kMineObject)
        owner->addToScore(100);
      else if (o->getType() == kHomingMineObject)
        owner->addToScore(200);
    }
		return kBothDestroyed;
  }
	//	printf("Distance to target: %1.2f\n", p1.length());
	
	return kNothingDestroyed;
}	

void Bullet::destroy()
{
	elapsedTime = 10.1;
}

void Bullet::setColor()
{
	glColor3f(1, 0, 0);
}