/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  GateKey.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Sat Apr 17 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "GateKey.h"

GateKey::GateKey(GLfloat x, GLfloat y, GLfloat z, GLfloat rad, WayGate *wg, double to, bool _open) // start coord (3)
:Object(kGatekeyObject, kMineObject|kHomingMineObject|kShipObject|kBulletObject), loc(x, y, z)
{
	door = wg;
	elapsedTime = to+1;
	radius = rad;
	rv = kObjectRemains;
	timeOut = to;
  open = _open;
  if (open)
    door->lock();
  else
    door->unlock();
}

GateKey::~GateKey()
{
}

point3d GateKey::getLocation()
{ return loc; }

void GateKey::draw()
{
  double detail = (gDetail)?16:4;
	glTranslatef(loc.x, loc.y, loc.z);
	setColor(); // orange?!?
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0, 0, 0);
	for (double x = 0; x <= TWOPI+.001; x += TWOPI/detail)
	{
		double xoff = cos(x);
		double yoff = sin(x);
		glVertex3f(xoff*radius, yoff*radius, 0);
	}
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0, 0, 0);
	for (double x = 0; x <= TWOPI+.001; x += TWOPI/detail)
	{
		double xoff = cos(x);
		double yoff = sin(x);
		glVertex3f(0, xoff*radius, yoff*radius);
	}
	glEnd();
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glTranslatef(-loc.x, -loc.y, -loc.z);
}

uint32_t GateKey::stepTime(double amount)
{
	elapsedTime += amount;
	if (elapsedTime > timeOut)
	{
    if (open)
      door->lock();
    else
      door->unlock();
	}
	return rv;
}

double GateKey::getRadius()
{
	return radius;
}

void GateKey::setColor()
{
  if (elapsedTime < timeOut)
    glColor3f(.5*elapsedTime/timeOut+.5, .25*elapsedTime/timeOut+.25, 0); // orange?!?
  else
    glColor3f(1, .5, 0); // orange?!?
}

uint32_t GateKey::testCollision(Object *o)
{
	vector3d p1 = getLocation().getVector(o->getLocation());
	float rad = getRadius()+o->getRadius();
	if (p1.squaredLength() < rad*rad)
	{
		if (elapsedTime > timeOut)
		{
			int notes[4] = {0, 1, 60, 120};
			pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 101));
      if (open)
        door->unlock();
      else
        door->lock();
		}
		elapsedTime = 0;
	}
	return kNothingDestroyed;
}

void GateKey::destroy()
{
	// gate keys are indestructable?!?
	//rv = kObjectDestroyed;
}