/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  LevelLoader.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Sat Apr 17 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include <string.h>
#include "LevelLoader.h"
#include "Bullet.h"
#include "GateKey.h"
#include "HomingMine.h"
#include "Mine.h"
#include "Music.h"
#include "Stars.h"
#include "TextBox.h"
#include "Turret.h"
#include "WayGate.h"

LevelLoader::LevelLoader()
{
}

LevelLoader::~LevelLoader()
{
}

int LevelLoader::getNumLevels()
{
	char path[1024];
	int x = 0;
	while (1)
	{
		FILE *f;
		getLevelPath(x, path);
		f = fopen(path, "r");
		if (f)
		{
			fclose(f);
		}
		else
			return x;
		x++;
	}
}

YourShip *LevelLoader::loadLevel(int which, World *w)
{
	FILE *f;
	char path[1024];
	getLevelPath(which, path);
	f = fopen(path, "r");
	if (f)
	{
    YourShip *ship;
    ship = new YourShip();
    w->reset();
    w->addObject(ship);
    
    // read & skip description
		int lineLen, numLines;
		fscanf(f, "%d %d\n", &numLines, &lineLen);
    char line[lineLen+2]; // + carriage return & null
		for (int x = 0; x < numLines; x++)
		{
			fgets(line, lineLen+2, f);
		}

    Object *old=0, *o=0;
    bool goal = false;
		while (o = loadNextObject(f, ship, goal))
    {
      //printf("Creating new item\n");
      if (goal)
      {
        //printf("(goal)\n");
        if (old == 0)
          o->activate();
        else
          old->setNextGoal(o);
        old = o;
      }
      w->addObject(o);
    }
    if (old)
      old->setNextGoal(ship);
    else
      ship->activate();
    
    fclose(f);
    return ship;
  }
  return 0;
}

void LevelLoader::loadLevelDescription(int which, World *w)
{
	FILE *f;
	char path[1024];
	getLevelPath(which, path);
	f = fopen(path, "r");
	if (f)
	{
		int lineLen, numLines;
		fscanf(f, "%d %d\n", &numLines, &lineLen);
		char *desc = new char[lineLen*numLines+1];
    for (int x = 0; x < lineLen*numLines; x++)
      desc[x] = ' ';
		desc[lineLen*numLines] = 0;

    char line[lineLen+2]; // + carriage return & null
		for (int x = 0; x < numLines; x++)
		{
			fgets(line, lineLen+2, f);
      //printf("read \"%s\"\n", line);
      int lenread = strlen(line)-1;
      for (int y = 0; y < lenread; y++)
        desc[x*lineLen+lineLen/2-lenread/2+y] = line[y];
      //printf("\"%s\"\n", desc);
		}
		desc[lineLen*numLines] = 0;
		fclose(f);

		point3d a(-.5, (double)numLines/lineLen, 0), b(.5, 0, 0);
    Stars *s;
		w->reset();
		w->addObject(s = new Stars(6000));
    s->startRotating();
    if (which <= 0)
    {
      a.x = -.9;
      b.x = .9;
      if (which == -1)
        a.y *= 1.8;
      else
        a.y = 0;
    }
		w->addObject(new TextBox(desc, lineLen, a, b, 60*60, true, (which==0)||(which==-2))); // animate level 0
    delete [] desc;
    if (which >  0)
    {
      a.y = -.6;
      w->addObject(new TextBox("  CLICK TO BEGIN  ", 18, a, b, 60*60));
    }
	}
}

void LevelLoader::getLevelPath(int which, char *result)
{
	FSRef file;
	GetApplicationBundleFSRef(&file);
	CFURLRef f = CFURLCreateFromFSRef(0, &file);
	CFStringRef fstr = CFURLCopyFileSystemPath(f, kCFURLPOSIXPathStyle);
	char buffer[1024];
	if (CFStringGetCString(fstr,
												 buffer,
												 1024,
												 kCFStringEncodingASCII))
	{
		//printf("Got path %s\n", buffer);
    if (which == -2)
      sprintf(result, "%s/Contents/Resources/about.txt", buffer);
    else if (which == -1)
      sprintf(result, "%s/Contents/Resources/keys.txt", buffer);
    else
      sprintf(result, "%s/Contents/Resources/level%d.txt", buffer, which);
		//printf("going to open %s\n", result);
	}
}

long LevelLoader::GetApplicationBundleFSRef(FSRef * location)
{
	OSStatus err;
	ProcessSerialNumber psn;
	
	err = GetCurrentProcess(&psn);
	if (err != noErr) {
		printf("Error with: GetCurrentProcess\n");
		return err;
	}
	err = GetProcessBundleLocation (&psn, location);
	if (err != noErr) {
		printf("Error with: GetProcessBundleLocation\n");
		return err;
	}
	return noErr;
}

Object *LevelLoader::loadNextObject(FILE *f, YourShip *targ, bool &goal)
{
  // first try to read
  static WayGate *last = 0;
  char name[128];

  goal = false;
  while (1) {
    fgets(name, 128, f);
    name[strlen(name)-1] = 0; // omit carriage return
    //fscanf(f, "%s\n", name);
    if (feof(f)) // if an error or EOF, we're done
      return 0;
    if (ferror(f) != 0)
      return 0;

    if (strcmp(name, "Goal") == 0)
    {
      goal = true;
      continue;
    }
    if (name[0] == '#')
      continue;
    break;
  }
  
  if (strcmp(name, "Bullet") == 0)
  {
    //float param[6];
    //fscanf(f, "%f %f %f %f
    //return new Bullet(GLfloat *params, 0); // start coord (3), velocity(3)
  }
  if (strcmp(name, "GateKey") == 0)
  {
    float x, y, z, radius, timeout;
    int open;
    fscanf(f, "%f %f %f %f %f %d\n", &x, &y, &z, &radius, &timeout, &open);
    return new GateKey(x, y, z, radius, last, timeout, open); // start coord (3)
  }
  if (strcmp(name, "HomingMine") == 0)
  {
    float x, y, z;
    fscanf(f, "%f %f %f\n", &x, &y, &z);
    return new HomingMine(x, y, z, targ);
  }
  if (strcmp(name, "Mine") == 0)
  {
    float x, y, z;
    fscanf(f, "%f %f %f\n", &x, &y, &z);
    return new Mine(x, y, z, 0, 0, 0, 24*60*60);
  }
  if (strcmp(name, "Music") == 0)
  {
    int len, options, param1, param2, instr;
    Music *m;
    fscanf(f, "%d %d %d %d %d", &instr, &len, &options, &param1, &param2);
    int score[len];
    for (int x = 0; x < len-1; x++)
      fscanf(f, "%d", &score[x]);
    fscanf(f, "%d\n", &score[len-1]);
    m = new Music(score, len, options, (((param1&0xFFFF)<<16)|(param2&0xFFFF)), instr, false);
    targ->setMusic(m);
    return m;
  }
  if (strcmp(name, "Stars") == 0)
  {
    float x, y, z;
    int num;
    fscanf(f, "%f %f %f %d\n", &x, &y, &z, &num);
    return new Stars(x, y, z, num);
  }
  if (strcmp(name, "TextBox") == 0)
  {
    //return new TextBox(char *text, int charLine, point3d topLeft, point3d bottomRight, double duration=4, bool deform=true, bool scrolling = false); // start coord (3), velocity(3)
  }
  if (strcmp(name, "Turret") == 0)
  {
    float x, y, z, fireRate, shotSpeed, rad;
    int what;
    fscanf(f, "%f %f %f %f %f %d %f\n", &x, &y, &z, &fireRate, &shotSpeed, &what, &rad);
    return new Turret(x, y, z, targ, what?kBullet:kMine, .25, fireRate, shotSpeed, rad);
  }
  if (strcmp(name, "WayGate") == 0)
  {
    float x, y, z, rx, ry, rz, radius;
    int resolution;
    fscanf(f, "%f %f %f %f %f %f %f %d\n", &x, &y, &z, &rx, &ry, &rz, &radius, &resolution);
    return (last = new WayGate(x, y, z, rx, ry, rz, radius, resolution));
  }
  return 0;
}
