/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */

/*
 *  HighScores.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Fri Apr 16 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "HighScores.h"

HighScores::HighScores()
{
	load();
}

HighScores::~HighScores()
{
	save();
}

void HighScores::save()
{
	FILE *f = openFile("w+");
	if (f)
	{
		fwrite(scores, sizeof(score), 10, f);
		fclose(f);
	}
}

void HighScores::load()
{
	FILE *f = openFile("r");
	if (f)
	{
		fread(scores, sizeof(score), 10, f);
		fclose(f);
		for (int x = 0; x < 10; x++)
		{
			for (int y = 0; y < 3; y++)
				if (scores[x].name[y] == 0)
					scores[x].name[y] = ' ';
			scores[x].name[3] = 0;
		}
	}
}

FILE *HighScores::openFile(char *style)
{
	FSRef ref;
	FSFindFolder(kOnAppropriateDisk, kPreferencesFolderType, kCreateFolder, &ref);
	CFURLRef furl = CFURLCreateFromFSRef(0, &ref);
	CFStringRef fstr = CFURLCopyFileSystemPath(furl, kCFURLPOSIXPathStyle);
	char buffer[1024];
	char buffer2[1024];
	if (CFStringGetCString(fstr, buffer, 1024, kCFStringEncodingASCII))
	{
		sprintf(buffer2, "%s/SV Racer High Scores", buffer);
		//printf("Going to open %s\n", buffer2);
		FILE *f = fopen(buffer2, style);
		return f;
	}
	return 0;
}

bool HighScores::isHighScore(uint32_t value)
{
	for (int x = 0; x < 10; x++)
		if (scores[x].value < value)
			return true;
	return false;
}

void HighScores::addHighScore(char *name, uint32_t value)
{
	for (int x = 0; x < 10; x++)
	{
		if (scores[x].value < value)
		{
			for (int y = 9; y > x; y--)
			{
				scores[y] = scores[y-1];
			}
			scores[x].value = value;
			for (int y = 0; y < 3; y++)
				scores[x].name[y] = (name[y]==0)?' ':name[y];
			scores[x].name[3] = 0;
			break;
		}
	}
	save();
}

TextBox *HighScores::getHighScoreList()
{
	char list[15*(10+2)+1];
	point3d a(-.5, .9, 0), b(.5, -.9, 0);
	sprintf(list, "  HIGH SCORES  ---------------");
	for (int x = 0; x < 10; x++)
	{
		list[(x+2)*15+0] = '0'+((scores[x].value/1000000)%10);
		list[(x+2)*15+1] = '0'+((scores[x].value/100000)%10);
		list[(x+2)*15+2] = '0'+((scores[x].value/10000)%10);
		list[(x+2)*15+3] = '0'+((scores[x].value/1000)%10);
		list[(x+2)*15+4] = '0'+((scores[x].value/100)%10);
		list[(x+2)*15+5] = '0'+((scores[x].value/10)%10);
		list[(x+2)*15+6] = '0'+((scores[x].value/1)%10);
		list[(x+2)*15+7] = ' ';
		list[(x+2)*15+8] = ' ';
		list[(x+2)*15+9] = ' ';
		list[(x+2)*15+10] = ' ';
		list[(x+2)*15+11] = ' ';
		list[(x+2)*15+12] = scores[x].name[0];
		list[(x+2)*15+13] = scores[x].name[1];
		list[(x+2)*15+14] = scores[x].name[2];
	}
	list[15*(10+2)] = 0;
	return new TextBox(list, 15, a, b, 24*60*60);
}
