/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  HomingMine.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Sun Apr 11 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */
#include "Fragment.h"
#include "HomingMine.h"

enum {
	kExploding,
	kArmed,
	kInert
};
/*
HomingMine::HomingMine(GLfloat *params, Object *_target)
:Object(kCollidingObject)
{
	for (int x = 0; x < 6; x++)
		param[x] = params[x];
	elapsedTime = 0;
	state = kInert;
	//fragCount = 0;
	target = _target;
	fVel[0] = fVel[1] = fVel[2] = 0;
	deployedMusic = 0;
	//deathSound = 0;
}
*/
HomingMine::HomingMine(GLfloat posx, GLfloat posy, GLfloat posz, Object *_target)
:Object(kHomingMineObject, kShipObject)
{
	param[0] = posx;
	param[1] = posy;
	param[2] = posz;
	param[3] = param[4] = param[5] = 0;
	elapsedTime = 0;
	state = kInert;
	//fragCount = 0;
	target = _target;
	fVel[0] = fVel[1] = fVel[2] = 0;
	deployedMusic = 0;
	//deathSound = 0;
}

void HomingMine::arm()
{
  if (state == kInert)
  {
    state = kArmed;
		int notes[8] = {0, 15, 40, 15, 0, 15, 41, 15};
		pushCreatedObject(deployedMusic = new Music(notes, 8, kRepeat, 10000, 1));
		setTestAgainst(kHomingMineObject|kShipObject|kMineObject);
  }
}

void HomingMine::draw()
{
	if (state == kInert)
	{
		glColor3f(0, 1.0, 1.0);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		drawMine();
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	}
	else if (state == kArmed)
	{
		glColor3f(1.0, 1.0, 0.0);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		drawMine();
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glColor3f(0, 0, 0);
		drawMine();
	}
	
}

void HomingMine::drawMine()
{
	glBegin(GL_TRIANGLES);
	glVertex3f(param[0], param[1]+.05*.86602/2, param[2]);
	glVertex3f(param[0]-.05*.86602/2, param[1]-.05*.86602/2, param[2]);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]+.05/2);

	glVertex3f(param[0], param[1]+.05*.86602/2, param[2]);
	glVertex3f(param[0]-.05*.86602/2, param[1]-.05*.86602/2, param[2]);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]-.05/2);

	glVertex3f(param[0], param[1]+.05*.86602/2, param[2]);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]+.05/2);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]-.05/2);

	glVertex3f(param[0]-.05*.86602/2, param[1]-.05*.86602/2, param[2]);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]+.05/2);
	glVertex3f(param[0]+.05*.86602/2, param[1]-.05*.86602/2, param[2]-.05/2);
	glEnd();
}

uint32_t HomingMine::stepTime(double amount)
{
	elapsedTime += amount;
	if (state == kArmed)
	{
		vector3d p1 = getLocation().getVector(target->getLocation());
		//printf("Distance to target: %1.2f\n", p1.length());
		p1.normalize();
		fVel[0] = fVel[0]*(1-amount/2)+3.5*p1.x*amount;
		fVel[1] = fVel[1]*(1-amount/2)+3.5*p1.y*amount;
		fVel[2] = fVel[2]*(1-amount/2)+3.5*p1.z*amount;
		for (int x = 0; x < 3; x++)
		if (fVel[x] > 10)
			fVel[x] = 10;
	}
	else if (state == kExploding)
	{
		return kObjectDestroyed;//|kNewObjectsCreated;
	}
	param[0] += fVel[0]*amount;
	param[1] += fVel[1]*amount;
	param[2] += fVel[2]*amount;
	//if (m)
		//return kObjectRemains|kNewObjectsCreated;
	return kObjectRemains;
}

//GLfloat fVel[3];

point3d HomingMine::getLocation()
{
	point3d p;
	p.x = param[0];
	p.y = param[1];
	p.z = param[2];
	return p;
}

//Object *HomingMine::getNewObject()
//{
//	if (m)
//	{
//		deployedMusic = m;
//		m = 0;
//		return deployedMusic;
//	}
//	if (deathSound)
//	{
//		Music *tmp = deathSound;
//		deathSound = 0;
//		return tmp;
//	}
//
//	if (fragCount > 0)
//	{
//		fragCount--;
//		// location, speed, rotational velocity
//	}
//	return 0;
//}


uint32_t HomingMine::testCollision(Object *o)
{
	vector3d p1 = getLocation().getVector(o->getLocation());
	double sLen = p1.squaredLength();
	if ((o == target) && (state == kInert) && (sLen < 100)) // activate when target is 10 away
	{
		state = kArmed;
		int notes[8] = {0, 15, 40, 15, 0, 15, 41, 15};
		pushCreatedObject(deployedMusic = new Music(notes, 8, kRepeat, 10000, 1));
		setTestAgainst(kHomingMineObject|kShipObject|kMineObject);
	}
	else {
		float rad = getRadius()+o->getRadius();
		if (sLen < rad*rad)
		{
			if (o == target)
				printf("Destroyed Target\n");
			return kBothDestroyed;
		}
	}
	//	printf("Distance to target: %1.2f\n", p1.length());
	
	return kNothingDestroyed;
}	

void HomingMine::destroy()
{
	state = kExploding;
  if (activeGoal)
    nextGoal->activate();
	if (deployedMusic)
		deployedMusic->destroy();
	int notes[4] = {0, 1, 65, 120};
	pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 128*4));
	for (int x = 0; x < 8; x++)
		pushCreatedObject(new Fragment(param[0], param[1], param[2],
																	 rand01()*.1-.05, rand01()*.1-.05, rand01()*.1-.05,
																	 rand01()*360, rand01()*360));
}

void HomingMine::setColor()
{
	if (state == kInert)
		glColor3f(0, .5, .5);
	else
		glColor3f(1, 1, 0);
}