/*
 *  Object.h
 *  Carbon OpenGL
 *
 *  Created by Nathan Sturtevant on Tue Mar 23 2004.
 *  Copyright (c) 2004 NS Software. All rights reserved.
 *
 */

#include <Carbon/Carbon.h>
#include <OpenGL/gl.h>
#include <stdint.h>
#include <stack>
#include "point3d.h"

#ifndef OBJECT_H
#define OBJECT_H

#define TWOPI           6.283185307179586476925287
#define PI              3.141592653589793238462643
#define PID2            PI/2

enum {
	kFrontPlane = 0,
	kBackPlane = 255
};

enum {
	kDisplayObject = 0x01,
	kShipObject = 0x02,
	kBulletObject = 0x04,
	kMineObject = 0x08,
	kHomingMineObject = 0x10,
	kWaygateObject = 0x20,
	kGatekeyObject = 0x40
};

// stepTime return codes
enum {
  kObjectDestroyed = 0x1, // remove from simulation
  kObjectDisabled = 0x2,  // remove from simulation, but don't delete
  kObjectRemains = 0x4,    // no changes
	kNewObjectsCreated = 0x8 // OBSOLETE
};

// collision result codes
enum {
	kNothingDestroyed,
	kThisDestroyed,
	kThatDestroyed,
	kBothDestroyed
};

class Object {
public:
	Object(uint32_t objectType, uint32_t testWith);
	virtual ~Object();
	virtual int getDepthPlane() { return 0; };
	virtual void draw() { };
  virtual uint32_t stepTime(double amount) = 0;
	virtual point3d getLocation();
	virtual void setColor() { glColor3f(1, 1, 1); }
	virtual uint32_t testCollision(Object *);
	virtual void destroy();
	virtual double getRadius() { return 0; }
	virtual void activate() { activeGoal = true; }
	virtual void setNextGoal(Object *o) { nextGoal = o; }
	void pushCreatedObject(Object *o);
	Object *getNewObject();

	inline bool isGoal() { return activeGoal; }
  inline uint32_t getType() { return objectType; }
	inline bool testAgainst(uint32_t what) {  return (testWith&what); }
  inline void setTestAgainst(uint32_t what) { testWith = what; }
	double rand01();
protected:
	Object *nextGoal;
	bool activeGoal;
private:
		std::stack<Object *> createdObjects;
	uint32_t objectType, testWith;
};

extern int gMusic, gSound, gDetail;

#endif