/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  Turret.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Sat Apr 17 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "Turret.h"
#include "Music.h"
#include "vector3d.h"
#include "Bullet.h"
#include "Fragment.h"
#include "HomingMine.h"

Turret::Turret(GLfloat x, GLfloat y, GLfloat z, Object *targ, int what, GLfloat rad, GLfloat fRate, GLfloat sSpeed, GLfloat actRad)
:Object(kDisplayObject, 0), loc(x, y, z)
{
	elapsedTime = 0;
	lastShot = 0;
	radius = rad;
	turnRate = 0; // unused for now
	fireRate = fRate;
	errorRate = 0;//eRate; // ununsed for now
	shotSpeed = sSpeed;
	shotDamage = 1;
	health = 1;
	state = kScanning;
	target = targ;
  create = what;
  activationRadius = actRad*actRad;
  creatingGoalObjects = false;
}

Turret::~Turret()
{
}

void Turret::draw()
{
	glTranslatef(loc.x, loc.y, loc.z);
	setColor();
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glBegin(GL_LINES);
  int limit = gDetail?30:15;
	for (int x = 0; x < limit; x++)
	{
		double ran = rand01()*TWOPI;
		double xoff = cos(ran);
		double yoff = sin(ran);
		glVertex3f(0, 0, 0);
		glVertex3f(xoff*radius, yoff*radius, 0);
	}
	glEnd();
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glTranslatef(-loc.x, -loc.y, -loc.z);
}

uint32_t Turret::stepTime(double amount)
{
	elapsedTime += amount;
	vector3d p1 = getLocation().getVector(target->getLocation());
	if (state == kScanning)
	{
    float slen = p1.squaredLength();
    if (slen < activationRadius)
      state = kTracking;
	}
  
	if (activeGoal || ((state == kTracking) && (lastShot + fireRate < elapsedTime)))
	{
		int notes[4] = {0, 0, 60-22, 30};
		lastShot = elapsedTime;
    if (create == kBullet)
    {
      p1.normalize();
      pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 117));
      GLfloat param[6];
      param[0] = loc.x;
      param[1] = loc.y;
      param[2] = loc.z;
      param[3] = shotSpeed*p1.x;
      param[4] = shotSpeed*p1.y;
      param[5] = shotSpeed*p1.z;
      pushCreatedObject(new Bullet(param));
    }
    else {
      // creating mines
      if (creatingGoalObjects && (!activeGoal))
      {
        // wait until previous mine is destroyed before creating new one.
      }
      else if (shotSpeed > 0)
      {
        int notes[4] = {0, 0, 60, 58};
        HomingMine *m;
        pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 127+128*3));
        pushCreatedObject(m = new HomingMine(loc.x, loc.y, loc.z, target));
        m->arm();
        shotSpeed-=1;
        if (creatingGoalObjects)
        {
          activeGoal = false;
          m->activate();
          m->setNextGoal(this);
        }
      }
      else if (creatingGoalObjects)
      {
        nextGoal->activate();
        creatingGoalObjects = false;
      }
    }
	}
	
	if (state == kDead)
		return kObjectDestroyed;
	return kObjectRemains;
}

point3d Turret::getLocation()
{
	return loc;
}

double Turret::getRadius()
{
	return radius;
}

void Turret::setColor()
{
	if (state == kScanning)
		glColor3f(.5, .5, .5);//, .25+.75*health);
	else
		glColor3f(1, 1, 1);//, .25+.75*health);
}

uint32_t Turret::testCollision(Object *o)
{
//	if (o != target)
//		return kNothingDestroyed;
//	vector3d p1 = getLocation().getVector(o->getLocation());
//	float slen = p1.squaredLength();
//	if ((slen < activationRadius) && (state == kScanning))
//	{
//		state = kTracking;
//	}
	
	return kNothingDestroyed;
}

void Turret::destroy()
{
//	health-=shotDamage;
/*
	if (health < 0)
	{
		state = kDead;

		int notes[4] = {0, 1, 63, 180};
		pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 128*4));
		for (int x = 0; x < 15; x++)
			pushCreatedObject(new Fragment(loc.x, loc.y, loc.z,
																		 rand01()*.1-.05, rand01()*.1-.05, rand01()*.1-.05,
																		 rand01()*360, rand01()*360));
	}
 */
}

void Turret::activate()
{
  activeGoal = true;
  creatingGoalObjects = true;
  create = kMine;
}
