/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  YourShip.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Sun Apr 11 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "YourShip.h"
#include "Fragment.h"
#include "Bullet.h"

static const float kTol = 0.001;
static const float kRad2Deg = 180. / 3.1415927;
static const float kDeg2Rad = 3.1415927 / 180.;

YourShip::YourShip()
:Object(kShipObject, 0)
{
	fVel[0] = 0.0; fVel[1] = 0.0; fVel[2] = 0.0; 
	fRot[0] = 180.0; fRot[1] = 0.0; fRot[2] = 1.0; fRot[3] = 0.0;
	viewPos.x = viewPos.y = viewPos.z = 0;
	elapsedTime = 0;
	alive = true;
	numMines = 10;
	shotPower = 100;
	viewRotate = kTop;
  gameMusic = 0;
  currentScore = 0;
}

int YourShip::getScore()
{
	return currentScore;
}

void YourShip::addToScore(int score)
{
  currentScore += score;
}

int YourShip::getTime()
{
	if (!alive)
		return (int)(deathTime-2);
	if (elapsedTime > 2)
		return (int)(elapsedTime-2);
	return 0;
}

void YourShip::setupViewpoint()
{
	glRotatef(fRot[0], fRot[1], fRot[2], fRot[3]);
	glTranslated(-viewPos.x, -viewPos.y, -viewPos.z);
}

void YourShip::rotateOverviewpoint()
{
	viewRotate = (viewRotate+1)%3;
}

tViewpoint YourShip::getOverviewpoint()
{
	return viewRotate;
}

void YourShip::setupOverviewpoint()
{
	GLfloat saved[4];
	GLfloat rot[4] = {90, 0, 0, 0};
	for (int x = 0; x < 4; x++)
		saved[x] = fRot[x];
	switch (viewRotate)
		{
		case kTop:
			rot[1] = 1;
			addRotation(rot);
			break;
		case kSide:
			rot[0] = -90;
			rot[2] = 1;
			addRotation(rot);
			break;
		case kRear:
		default: break;
		}
	glRotatef(fRot[0], fRot[1], fRot[2], fRot[3]);
	glTranslated(-viewPos.x, -viewPos.y, -viewPos.z);
	for (int x = 0; x < 4; x++)
		fRot[x] = saved[x];
}

void YourShip::drawStatus()
{
  if (shotPower > 100)
    shotPower = 100;
  if (shotPower < 0)
    shotPower = 0;
  if (numMines > 10)
    numMines = 10;
  if (numMines < 0)
    numMines = 0;
	glDisable(GL_DEPTH_TEST);
	glColor3f(1, 0, 0);
	glBegin(GL_LINE_LOOP);
	glVertex3f(-.75+.05, -.8+1.4*shotPower/100.0, 0);
	glVertex3f(-.75+.05, -.8, 0);
	glVertex3f(-.45+.05, -.8, 0);
	glVertex3f(-.45+.05, -.8+1.4*shotPower/100.0, 0);
	glEnd();
//	glColor3f(1, 1, 1);
//	glBegin(GL_LINE_LOOP);
//	glVertex3f(-.7, .6, 0);
//	glVertex3f(-.7, -.8, 0);
//	glVertex3f(-.2, -.8, 0);
//	glVertex3f(-.2, .6, 0);
//	glEnd();
	glBegin(GL_LINES);
	glVertex3f(-.75+.05, -.8+.14, 0);
	glVertex3f(-.45+.05, -.8+.14, 0);
	glEnd();
	
	glColor3f(0, 0, 1);
	glBegin(GL_LINE_LOOP);
	glVertex3f(-.15, -.8+1.4*numMines/10.0, 0);
	glVertex3f(-.15, -.8, 0);
	glVertex3f(.15, -.8, 0);
	glVertex3f(.15, -.8+1.4*numMines/10.0, 0);
	glEnd();	
//	glColor3f(1, 1, 1);
//	glBegin(GL_LINE_LOOP);
//	glVertex3f(.2, .6, 0);
//	glVertex3f(.2, -.8, 0);
//	glVertex3f(.7, -.8, 0);
//	glVertex3f(.7, .6, 0);
//	glEnd();
  double norm = sqrt(fVel[0]*fVel[0]+fVel[1]*fVel[1]+fVel[2]*fVel[2])/10;
	glColor3f(0, 1, 0);
	glBegin(GL_LINE_LOOP);
	glVertex3f(.45-.05, -.8+1.4*norm, 0);
	glVertex3f(.45-.05, -.8, 0);
	glVertex3f(.75-.05, -.8, 0);
	glVertex3f(.75-.05, -.8+1.4*norm, 0);
	glEnd();	

	glEnable(GL_DEPTH_TEST);
}

bool YourShip::passedLevel()
{
	if (!activeGoal)
		return false;
	else if (elapsedTime > deathTime+3)
		return true;
	return false;
}

void YourShip::activate()
{
	if (gameMusic)
    gameMusic->destroy();
	gameMusic = 0;
	activeGoal = true;
	deathTime = elapsedTime;
	fVel[0] = heading.x;
	fVel[1] = heading.y;
	fVel[2] = heading.z;
	viewPos.x += 5*heading.x;
	viewPos.y += 5*heading.y;
	viewPos.z += 5*heading.z;
	GLfloat rot[4] = {90, 0, 1, 0};
	addRotation(rot);
				
	point3d a(-.5, .2, 1), b(.5, -.2, 1);
	pushCreatedObject(new TextBox("  FINISHED LEVEL  ", 18, a, b, 3));
}


bool YourShip::isAlive()
{
	if (alive)
		return true;
	else {
		if (elapsedTime > deathTime+2)
			return false;
	}
	return true;
}

uint32_t YourShip::stepTime(double amount)
{
	elapsedTime += amount;
	shotPower += 10*amount;
	if (shotPower > 100)
		shotPower = 100;
	getHeading(heading.x, heading.y, heading.z);
	handleKeyboard(30.0*amount, 2.5*amount, pow(.95, 30*amount));
	viewPos.x += fVel[0]*amount;
	viewPos.y += fVel[1]*amount;
	viewPos.z += fVel[2]*amount;

	if (!alive)
	{
		if (elapsedTime > deathTime+2)
			return kObjectDisabled;
		return kObjectRemains;
	}
	return kObjectRemains;
}

void YourShip::draw()
{
}

void YourShip::handleKeyboard(double rotation, double speed, double stop)
{
	if ((!alive) || (activeGoal))
		return;
	enum {
		kW = 21, // forward
		kS = 25, // backward
		kA = 24, // rotate CCW
		kD = 26, // rotate CW
		kSpace = 41, // stop
	};
	if (isPressed(kD))
	{
		float rote[4] = {0.0f, 0.0f, 0.0f, 0.0f};
		rote[0] = rotation;
		rote[3] = 1;
		addRotation(rote);
	}
	if (isPressed(kA))
	{
		float rote[4] = {0.0f, 0.0f, 0.0f, 0.0f};
		rote[0] = -rotation;
		rote[3] = 1;
		addRotation(rote);
	}
	if (isPressed(kW))
	{
		fVel[0]-=speed*heading.x;
		fVel[1]-=speed*heading.y;
		fVel[2]-=speed*heading.z;
		double norm = (fVel[0]*fVel[0]+fVel[1]*fVel[1]+fVel[2]*fVel[2])/100;
		if (norm > 1)
		{
			norm = 1/sqrt(norm);
			fVel[0]*=norm;
			fVel[1]*=norm;
			fVel[2]*=norm;
		}
	}
	if (isPressed(kS))
	{
		fVel[0]+=speed*heading.x;
		fVel[1]+=speed*heading.y;
		fVel[2]+=speed*heading.z;
		double norm = (fVel[0]*fVel[0]+fVel[1]*fVel[1]+fVel[2]*fVel[2])/100;
		if (norm > 1)
		{
			norm = 1/sqrt(norm);
			fVel[0]*=norm;
			fVel[1]*=norm;
			fVel[2]*=norm;
		}
	}
	if (isPressed(kSpace))
	{
		fVel[0] *= stop;
		fVel[1] *= stop;
		fVel[2] *= stop;
	}		
}

void YourShip::getHeading(GLfloat &hx, GLfloat &hy, GLfloat &hz)
{
	// these formulas are derived from the opengl redbook 1.4 pg 700
	double xp, yp, zp, len, sa, ca;//hx, hy, hz,
	len = 1/sqrt(fRot[1]*fRot[1] +
							 fRot[2]*fRot[2] +
							 fRot[3]*fRot[3]);
	xp = fRot[1]*len;
	yp = fRot[2]*len;
	zp = fRot[3]*len;
	ca = cos(-fRot[0]*PI/180);
	sa = sin(-fRot[0]*PI/180);
	hx = (1-ca)*xp*zp+sa*yp;
	hy = (1-ca)*yp*zp-sa*xp;
	hz = ca+(1-ca)*zp*zp;
	len = 1/sqrt(hx*hx+hy*hy+hz*hz);
	hx *= len;
	hy *= len;
	hz *= len;
	//printf("Heading vector: (%1.3f, %1.3f, %1.3f)\n", hx, hy, hz);
}

void YourShip::addRotation(float * dA)
{
	if ((!alive) || (activeGoal))
		return;
	float *A = fRot;
	float q0[4], q1[4], q2[4];
	float theta2, sinTheta2;
	
	// Figure out A' = A . dA
	// In quaternions: let q0 <- A, and q1 <- dA.
	// Figure out q2 = q1 + q0 (note the order reversal!).
	// A' <- q3.
	
	rotation2Quat(A, q0);
	rotation2Quat(dA, q1);
	
	// q2 = q1 + q0;
	q2[0] = q1[1]*q0[2] - q1[2]*q0[1] + q1[3]*q0[0] + q1[0]*q0[3];
	q2[1] = q1[2]*q0[0] - q1[0]*q0[2] + q1[3]*q0[1] + q1[1]*q0[3];
	q2[2] = q1[0]*q0[1] - q1[1]*q0[0] + q1[3]*q0[2] + q1[2]*q0[3];
	q2[3] = q1[3]*q0[3] - q1[0]*q0[0] - q1[1]*q0[1] - q1[2]*q0[2];
	// Here's an excersize for the reader: it's a good idea to re-normalize your quaternions
	// every so often.  Experiment with different frequencies.
	
	// An identity rotation is expressed as rotation by 0 about any axis.
	// The "angle" term in a quaternion is really the cosine of the half-angle.
	// So, if the cosine of the half-angle is one (or, 1.0 within our tolerance),
	// then you have an identity rotation.
	if (fabs(fabs(q2[3] - 1.)) < 1.0e-7) {
		// Identity rotation.
		A[0] = 0.0f;
		A[1] = 1.0f;
		A[2] = A[3] = 0.0f;
		return;
	}
	
	// If you get here, then you have a non-identity rotation.  In non-identity rotations,
	// the cosine of the half-angle is non-0, which means the sine of the angle is also
	// non-0.  So we can safely divide by sin(theta2).
	
	// Turn the quaternion back into an {angle, {axis}} rotation.
	theta2 = (float) acos (q2[3]);
	sinTheta2 = (float)  (1.0 /  sin ((double) theta2));
	A[0] = theta2 * 2.0f * kRad2Deg;
	A[1] = q2[0] * sinTheta2;
	A[2] = q2[1] * sinTheta2;
	A[3] = q2[2] * sinTheta2;
}

void YourShip::rotation2Quat (float *A, float *q)
{
	float ang2;  // The half-angle
	float sinAng2; // sin(half-angle)
	
	// Convert a GL-style rotation to a quaternion.  The GL rotation looks like this:
	// {angle, x, y, z}, the corresponding quaternion looks like this:
	// {{v}, cos(angle/2)}, where {v} is {x, y, z} / sin(angle/2).
	
	ang2 = A[0] * kDeg2Rad * 0.5;  // Convert from degrees ot radians, get the half-angle.
	sinAng2 = sin(ang2);
	q[0] = A[1] * sinAng2; q[1] = A[2] * sinAng2; q[2] = A[3] * sinAng2;
	q[3] = cos(ang2);
}


void YourShip::dropMine()
{
	if ((!alive) || (activeGoal))
		return;
	if (numMines > 0)
	{
		int notes[4] = {0, 0, 60, 60};
		pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 127+128*3));
		numMines--;
		GLfloat param[6];
		GLfloat dir = 10;
		param[0] = viewPos.x;
		param[1] = viewPos.y;
		param[2] = viewPos.z;
		param[3] = dir*heading.x+fVel[0];
		param[4] = dir*heading.y+fVel[1];
		param[5] = dir*heading.z+fVel[2];
		pushCreatedObject(new Mine(param));
	}
}

void YourShip::shoot()
{
	if ((!alive) || (activeGoal))
		return;
	if (shotPower > 10)
	{
		int notes[4] = {0, 0, 60-24, 30};
		pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 117));
		shotPower -= 10;
		GLfloat param[6];
		GLfloat dir = -15;
		param[0] = viewPos.x;
		param[1] = viewPos.y;
		param[2] = viewPos.z;
		param[3] = dir*heading.x+fVel[0];
		param[4] = dir*heading.y+fVel[1];
		param[5] = dir*heading.z+fVel[2];
		pushCreatedObject(new Bullet(param, this));
		//printf("bang\n");
	}
}

bool YourShip::isPressed(unsigned short k)
// k =  any keyboard scan code, 0-127
{
	unsigned long km[4];
	
	GetKeys( (long *) km );
	return (km[k>>5]>>(k&0x1F))&0x1;
}

uint32_t YourShip::testCollision(Object *o)
{
	vector3d p1 = getLocation().getVector(o->getLocation());
	float rad = getRadius()+o->getRadius();
	if (p1.squaredLength() < rad*rad)
	{
		return kThisDestroyed;
	}
//	printf("Distance to target: %1.2f\n", p1.length());

	return kNothingDestroyed;
	//	kThisDestroyed,
	//	kThatDestroyed,
	//	kBothDestroyed
}

void YourShip::destroy()
{
	if (activeGoal)
		return;
	//printf("Your ship was destroyed\n");
	if (alive)
	{
		alive = false;

		for (int x = 0; x < 20; x++)
			pushCreatedObject(new Fragment(viewPos.x, viewPos.y, viewPos.z,
																		 -heading.x+rand01()*.1-.05, -heading.y+rand01()*.1-.05, -heading.z+rand01()*.1-.05,
																		 rand01()*360, rand01()*360));
		int notes[38] = {0, 15, 64, 15, 64, 0, 0, 15, 64, 15, 64, 0,
														63, 15, 63, 0, 0, 15, 63, 15, 63, 0,
														62, 15, 62, 0, 0, 15, 62, 15, 62, 0,
														61, 90, 61, 0};
		pushCreatedObject(new Music(notes, 38, kPlayOnce, 0, 87));
		fVel[0] = fVel[1] = fVel[2] = 0;
    if (gameMusic)
      gameMusic->destroy();
		gameMusic = 0;
		point3d a(-.5, 1.0/14.0, 0.5), b(.5, -1.0/14.0, 0.5);
		pushCreatedObject(new TextBox("SHIP DESTROYED", 14, a, b, 60));
		deathTime = elapsedTime;
		int note[4] = {0, 0, 55, 60*5};
		pushCreatedObject(new Music(note, 4, kPlayOnce, 0, (128*4)+126));

		// step back so we can user can watch our pieces fly away
		viewPos.x += 2*heading.x;
		viewPos.y += 2*heading.y;
		viewPos.z += 2*heading.z;
	}
}

void YourShip::setColor()
{
	glColor3f(1, 1, 1);
}
